@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Device::Chip::Adapter );
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 use Carp;
 
@@ -17,26 +17,28 @@ use Test::Future::Deferred;
 use List::Util 1.33 qw( first any );
 use Test::Builder;
 
+use Test::ExpectAndCheck::Future;
+
 =head1 NAME
 
 C<Test::Device::Chip::Adapter> - unit testing on C<Device::Chip>
 
 =head1 SYNOPSIS
 
- use Test::More;
- use Test::Device::Chip::Adapter;
+   use Test::More;
+   use Test::Device::Chip::Adapter;
 
- my $adapter = Test::Device::Chip::Adapter->new;
+   my $adapter = Test::Device::Chip::Adapter->new;
 
- $chip_under_test->mount( $adapter );
+   $chip_under_test->mount( $adapter );
 
- # An actual test
- $adapter->expect_readwrite( "123" )
-    ->returns( "45" );
+   # An actual test
+   $adapter->expect_readwrite( "123" )
+      ->returns( "45" );
 
- is( $chip->do_thing( "123" )->get, "45", 'result of ->do_thing' );
+   is( $chip->do_thing( "123" )->get, "45", 'result of ->do_thing' );
 
- $adapter->check_and_clear( '->do_thing' );
+   $adapter->check_and_clear( '->do_thing' );
 
 =head1 DESCRIPTION
 
@@ -55,9 +57,13 @@ correctly perform the required asynchronisation.
 sub new
 {
    my $class = shift;
+
+   my ( $controller, $obj ) = Test::ExpectAndCheck::Future->create;
+
    return bless {
       builder => Test::Builder->new,
-      expect  => [],
+      controller => $controller,
+      obj        => $obj,
    }, $class;
 }
 
@@ -94,27 +100,6 @@ sub configure
    Test::Future::Deferred->done_later;
 }
 
-sub _stringify
-{
-   my ( $v ) = @_;
-   if( $v =~ m/^-?[0-9]+$/ ) {
-      return sprintf "0x%X", $v;
-   }
-   elsif( $v =~ m/^[\x20-\x7E]*$/ ) {
-      $v =~ s/([\\'])/\\$1/g;
-      return qq('$v');
-   }
-   else {
-      $v =~ s{(.)}{sprintf "\\x%02X", ord $1}gse;
-      return qq("$v");
-   }
-}
-
-sub _stringify_args
-{
-   join ", ", map { _stringify $_ } @_;
-}
-
 =head1 EXPECTATIONS
 
 Each of the actual methods to be used by the L<Device::Chip> under test has an
@@ -122,23 +107,23 @@ associated expectation method, whose name is prefixed C<expect_>. Each returns
 an expectation object, which has additional methods to control the behaviour of
 that invocation.
 
- $exp = $adapter->expect_write_gpios( \%gpios )
- $exp = $adapter->expect_read_gpios( \@gpios )
- $exp = $adapter->expect_tris_gpios( \@gpios )
- $exp = $adapter->expect_write( $bytes )
- $exp = $adapter->expect_read( $len )
- $exp = $adapter->expect_write_then_read( $bytes, $len )
- $exp = $adapter->expect_readwrite( $bytes_out )
- $exp = $adapter->expect_assert_ss
- $exp = $adapter->expect_release_ss
- $exp = $adapter->expect_readwrite_no_ss( $bytes_out )
- $exp = $adapter->expect_write_no_ss( $bytes )
+   $exp = $adapter->expect_write_gpios( \%gpios )
+   $exp = $adapter->expect_read_gpios( \@gpios )
+   $exp = $adapter->expect_tris_gpios( \@gpios )
+   $exp = $adapter->expect_write( $bytes )
+   $exp = $adapter->expect_read( $len )
+   $exp = $adapter->expect_write_then_read( $bytes, $len )
+   $exp = $adapter->expect_readwrite( $bytes_out )
+   $exp = $adapter->expect_assert_ss
+   $exp = $adapter->expect_release_ss
+   $exp = $adapter->expect_readwrite_no_ss( $bytes_out )
+   $exp = $adapter->expect_write_no_ss( $bytes )
 
 The returned expectation object allows the test script to specify what such an
 invocation should return or throw.
 
- $exp->returns( $bytes_in )
- $exp->fails( $failure )
+   $exp->returns( $bytes_in )
+   $exp->fails( $failure )
 
 =cut
 
@@ -175,10 +160,7 @@ foreach my $method ( keys %METHODS ) {
       my $self = shift;
       @_ = $canonicalise->( @_ ) if $canonicalise;
 
-      push @{ $self->{expect} }, my $e = bless [ $method, [ @_ ], 0 ],
-         "Test::Device::Chip::Adapter::_Expectation";
-
-      return $e;
+      return $self->{controller}->expect( $method => @_ );
    };
 
    my $actual = sub {
@@ -190,13 +172,7 @@ foreach my $method ( keys %METHODS ) {
       any { $_ eq $self->{protocol} } @$allowed_protos or
          croak "Method ->$method not allowed in $self->{protocol} protocol";
 
-      my $e = first { not $_->complete } @{ $self->{expect} };
-
-      $e and $e->consume( $method, \@args ) or
-         # TODO: reflect this in test result
-         croak "Unexpected call to ->$method(${\ _stringify_args @args })";
-
-      return $e->future;
+      return $self->{obj}->$method( @args );
    };
 
    no strict 'refs';
@@ -227,80 +203,8 @@ sub check_and_clear
    my $self = shift;
    my ( $name ) = @_;
 
-   my $builder = $self->{builder};
-
-   $builder->subtest( $name, sub {
-      my $count = 0;
-      foreach my $e ( @{ $self->{expect} } ) {
-         $e->check( $builder );
-         $count++;
-      }
-
-      $builder->ok( 1, "No calls made" ) if !$count;
-   });
-
-   $self->{expect} = [];
-}
-
-package
-   Test::Device::Chip::Adapter::_Expectation;
-
-use List::Util qw( all );
-
-use constant {
-   METHOD  => 0,
-   ARGS    => 1,
-   CALLED  => 2,
-   RETURN  => 3,
-   FAILURE => 4,
-};
-
-sub complete
-{
-   my $self = shift;
-   return $self->[CALLED];
-}
-
-sub consume
-{
-   my $self = shift;
-   my ( $method, $args ) = @_;
-
-   $method eq $self->[METHOD] or return 0;
-   @$args == @{ $self->[ARGS] } or return 0;
-   all { $args->[$_] eq $self->[ARGS][$_] } 0 .. $#$args or return 0;
-
-   $self->[CALLED]++;
-   return 1;
-}
-
-sub check
-{
-   my $self = shift;
-   my ( $builder ) = @_;
-
-   $builder->ok( $self->[CALLED], $self->[METHOD] );
-}
-
-sub returns
-{
-   my $self = shift;
-   $self->[FAILURE] and die "Cannot set a return value for a failing expectation\n";
-   $self->[RETURN] = [ @_ ];
-}
-
-sub fails
-{
-   my $self = shift;
-   $self->[RETURN] and die "Cannot set a failure for a returning expectation\n";
-   $self->[FAILURE] = [ @_ ];
-}
-
-sub future
-{
-   my $self = shift;
-   $self->[FAILURE] ? Test::Future::Deferred->fail_later( @{ $self->[FAILURE] } )
-                    : Test::Future::Deferred->done_later( @{ $self->[RETURN] } )
+   $self->{controller}->check_and_clear( $name );
+   return;
 }
 
 =head1 TODO
